@@ -17,10 +17,8 @@ call.
  *                                                                         *
  ***************************************************************************/
 
-#include <unistd.h>
 #include <signal.h>
 #include "sipomatic.h"
-#include <glib.h>
 
 
 int run_cond=1;
@@ -143,9 +141,10 @@ int payload_is_supported(SdpPayload *payload,RtpProfile *local_profile,RtpProfil
 		PayloadType *rtppayload;
 		rtppayload=rtp_profile_get_payload(local_profile,localpt);
 		if (rtppayload==NULL) return 0;
-		rtp_profile_set_payload(dialog_profile,payload->pt,payload_type_clone(rtppayload));
+		rtppayload=payload_type_clone(rtppayload);
+		rtp_profile_set_payload(dialog_profile,payload->pt,rtppayload);
 		/* add to the rtp payload type some other parameters (bandwidth) */
-		if (payload->b_as_bandwidth!=0) rtppayload->normal_bitrate=payload->b_as_bandwidth;
+		if (payload->b_as_bandwidth!=0) rtppayload->normal_bitrate=payload->b_as_bandwidth*1000;
 		return 1;
 	}
 	return 0;
@@ -192,7 +191,7 @@ void sipomatic_init(Sipomatic *obj, gchar *url)
 	BodyHandler *sdph;
 	if (url==NULL){
 		url=getenv("SIPOMATIC_URL");
-		if (url==NULL) url="sip:robot@127.0.0.1:5064";
+		if (url==NULL) url="sip:robot@0.0.0.0:5064";
 	}
 	g_message("Starting using url %s",url);
 	obj->audio_codecs=ms_codec_get_all_audio();
@@ -349,7 +348,7 @@ int main(int argc, char *argv[])
 			continue;
 		}
 		if ( (strcmp(argv[i],"-v")==0) || (strcmp(argv[i],"--version")==0) ){
-			printf("version: " VERSION "\n");
+			printf("version: " LINPHONE_VERSION "\n");
 			exit(0);
 		}
 		if (strcmp(argv[i],"-u")==0){
@@ -380,7 +379,7 @@ int main(int argc, char *argv[])
 	rtp_profile_set_payload(&av_profile,110,&speex_nb);
 	rtp_profile_set_payload(&av_profile,111,&speex_wb);
 	rtp_profile_set_payload(&av_profile,101,&telephone_event);
-        rtp_profile_set_payload(&av_profile,116,&truespeech);
+ 	rtp_profile_set_payload(&av_profile,116,&truespeech);
 	TRACE_INITIALIZE(TRACE_LEVEL6,stdout);
 	osipua_init();
 	osip_manager_set_send_port(def_manager,sendport);
