@@ -24,12 +24,18 @@
  */
 
 #include <config.h>
-#include <glib.h>
 
-#include <string.h>
+
+
+#ifdef HAVE_GLIB
+#include <glib.h>
+#else
+#include <uglib.h>
+#endif
+
+
 #include <stdio.h>
 #include <time.h>
-#include <string.h>
 #include <unistd.h>	/* unlink() */
 #include <stdlib.h>	/* atoi() */
 #include <sys/types.h>
@@ -111,7 +117,7 @@ static char *gnome_user_accels_dir = NULL;
 const char *
 gnome_user_dir_get (void)
 {
-	return g_build_filename (g_get_home_dir(), GNOME_DOT_GNOME, NULL);;
+	return g_build_filename (g_get_home_dir(), GNOME_DOT_GNOME, NULL);
 }
 
 /**
@@ -129,7 +135,7 @@ const char *
 gnome_user_private_dir_get (void)
 {
 	return g_build_filename (g_get_home_dir(),
-							   GNOME_DOT_GNOME_PRIVATE, NULL);;
+							   GNOME_DOT_GNOME_PRIVATE, NULL);
 }
 
 /**
@@ -488,6 +494,10 @@ new_key (TSecHeader *section, const char *key_name, const char *value)
 	TKeys *key;
     
 	key = (TKeys *) g_malloc (sizeof (TKeys));
+	if (key_name==NULL)
+		g_error("Key name is null for value %s",value);
+	if (value==NULL)
+		g_error("Value is null for key %s",key_name);
 	key->key_name = g_strdup (key_name);
 	key->value   = g_strdup (value);
 	key->link = section->keys;
@@ -725,7 +735,7 @@ check_path(char *path, mode_t newmode)
 {
 	gchar *dirpath;
 	gchar *p, *cur;
-	GString *newpath;
+	gchar *newpath;
 	struct stat s;
 
 	g_return_val_if_fail (path != NULL, FALSE);
@@ -763,20 +773,17 @@ check_path(char *path, mode_t newmode)
 		p++;
 
 	cur = p;
-	newpath = g_string_new("");
 	while ((p = cur)) {
 	       	cur = strchr (cur, '/');
 		if (cur) {
 			*cur = '\0';
 			cur++;
 		}
-		
-		newpath = g_string_append_c(newpath,'/');
-		newpath = g_string_append(newpath,p);
-		if(stat(newpath->str,&s)==0) {
+    newpath=g_strdup_printf("/%s",p);
+		if(stat(newpath,&s)==0) {
 			/*check if a directory*/
 			if(!S_ISDIR(s.st_mode)) {
-				g_string_free(newpath,TRUE);
+				g_free(newpath);
 				return FALSE;
 			}
 		} else {
@@ -784,16 +791,14 @@ check_path(char *path, mode_t newmode)
 			  directory*/
 			//g_warning("check_path: could not stat %s",newpath->str);
 
-			if(mkdir(newpath->str,newmode)!=0) {
+			if(mkdir(newpath,newmode)!=0) {
 				/*error, return false*/
-				g_string_free(newpath,TRUE);
+				g_free(newpath);
 				return FALSE;
 			}
 		}
 	}
-
-	g_string_free(newpath,TRUE);
-
+  g_free(newpath);
 	return TRUE;
 }
 
@@ -1835,7 +1840,7 @@ gnome_config_get_bool_with_default_ (const char *path, gboolean *def,
 		return 0;
 	}
 
-	if (g_ascii_tolower(*r) == 't' || g_ascii_tolower(*r) == 'y' || atoi(r)) {
+	if (*r == 't' || *r == 'y' || atoi(r) || *r == 'T' || *r == 'Y') {
 	  v = 1;
 	} else {
 	  /* If it's not true it has to be false :) */
