@@ -20,13 +20,14 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #include "linphonecore.h"
 #include <mediastream.h>
 #include <osipua.h>
-#include <sdphandler.h>
+#include "../osipua/src/sdphandler.h"
 #include "gnome-config.h"
 #include "misc.h"
 
 #include <telephonyevents.h>
 #include <utils.h>
 #include "osipuacb.h"
+#include "enum.h"
 
 #include <net/if.h>
 #include <sys/ioctl.h>
@@ -49,9 +50,10 @@ const double bandwidths[5]=
 };/* carefull: these are the upstream bandwidth !!!*/
 
 /* relative path where is stored local ring*/
-#define LOCAL_RING "ring.wav"
+#define LOCAL_RING "rings/oldphone.wav"
 /* same for remote ring (ringback)*/
-#define REMOTE_RING "ringback.wav"
+#define REMOTE_RING_FR "ringback.wav"
+#define REMOTE_RING_US "ringback.wav"
 
 
 /* these are useful messages for the status */
@@ -62,6 +64,18 @@ gchar *connected;
 gchar *cancel;
 gchar *contacted;
 
+
+void lc_callback_obj_init(LCCallbackObj *obj,LinphoneCoreCbFunc func,gpointer ud)
+{
+  obj->_func=func;
+  obj->_user_data=ud;
+}
+
+gint lc_callback_obj_invoke(LCCallbackObj *obj, LinphoneCore *lc){
+  if (obj->_func!=NULL) obj->_func(lc,obj->_user_data);
+  return 0;
+}
+
 LinphoneDialogParams * linphone_dialog_params_new()
 {
 	return g_new0(LinphoneDialogParams,1);
@@ -285,10 +299,12 @@ void find_best_interface(net_config_t *config)
 #endif /*LINPHONE_DEPRECATED*/
 
 void
-net_config_init (net_config_t * config)
+net_config_read (LinphoneCore *lc)
 {
 	gchar *ifname;
 	gboolean def=FALSE;
+	gchar *tmpstr;
+	gint tmp;
 	
 #ifdef LINPHONE_DEPRECATED
 	config->interfaces=retrieve_if();
@@ -307,24 +323,29 @@ net_config_init (net_config_t * config)
 	}else find_best_interface(config);
 #endif /*LINPHONE_DEPRECATED*/	
 
-	config->con_type=gnome_config_get_int_with_default("net/con_type",&def);
-	if (def) config->con_type=CONNECTION_TYPE_56KMODEM;
-	config->bandwidth=bandwidths[config->con_type];
+	tmp=gnome_config_get_int_with_default("net/con_type",&def);
+#ifdef __ARM__
+	if (def) tmp=CONNECTION_TYPE_ETHERNET;
+#else
+	if (def) tmp=CONNECTION_TYPE_56KMODEM;
+#endif
+	linphone_core_set_connection_type(lc,tmp);
 	
-	config->nat_address=gnome_config_get_string_with_default("net/nat_address",&def);
-	if (def) config->nat_address=NULL;
-	if (config->nat_address!=NULL && (strlen(config->nat_address)<1)) config->nat_address=NULL;
-	config->use_nat=gnome_config_get_int_with_default("net/use_nat",&def);
-	if (def) config->use_nat=0;	
+	tmpstr=gnome_config_get_string_with_default("net/nat_address",&def);
+	if (def) tmpstr=NULL;
+	if (tmpstr!=NULL && (strlen(tmpstr)<1)) tmpstr=NULL;
+	tmp=gnome_config_get_int_with_default("net/use_nat",&def);
+	if (def) tmp=0;
+	linphone_core_set_nat_address(lc,tmpstr,tmp);		
 }
 
-void sound_config_init(sound_config_t *config)
+void sound_config_read(LinphoneCore *lc)
 {
 	gboolean def;
-	int tmp;
+	int devid,tmp;
 	char *tmpbuf;
 	SndCard *sndcard;
-	config->dev_id=gnome_config_get_int_with_default("sound/dev_id",&def);
+	devid=gnome_config_get_int_with_default("sound/dev_id",&def);
 	if (def) {
 		/* choose an oss card if availlable, because alsa native support is not well enough stable*/
 		int i;
@@ -332,121 +353,124 @@ void sound_config_init(sound_config_t *config)
 		for(i=0;i<MAX_SND_CARDS;i++){
 			sndcard=snd_card_manager_get_card(snd_card_manager,i);
 			if (sndcard==NULL){
-				config->dev_id=0;
+				devid=0;
 				break;
 			}
 			id=snd_card_get_identifier(sndcard);
 			if (strstr(id,"Open Sound System")!=0){
 				g_message("Choosing oss device %s.",id);
-				config->dev_id=i;
+				devid=i;
 				break;
 			}
 		}
 	}
-	config->play_lev=gnome_config_get_int_with_default("sound/play_lev",&def);
-	if (def) config->play_lev=80;
-	config->rec_lev=gnome_config_get_int_with_default("sound/rec_lev",&def);
-	if (def) config->rec_lev=80;
+	linphone_core_set_sound_device(lc,devid);
+	tmp=gnome_config_get_int_with_default("sound/play_lev",&def);
+	if (def) tmp=80;
+	linphone_core_set_play_level(lc,tmp);
+	tmp=gnome_config_get_int_with_default("sound/rec_lev",&def);
+	if (def) tmp=80;
+	linphone_core_set_rec_level(lc,tmp);
 	tmpbuf=gnome_config_get_string_with_default("sound/source",&def);
-	if (def) config->source='m';
-	else {
-		config->source=tmpbuf[0];
-	}
-	config->local_ring=gnome_config_get_string_with_default("sound/local_ring",&def);
-	if (def) config->local_ring=PACKAGE_SOUND_DIR "/" LOCAL_RING;
-	else {
-		if (!g_file_test(config->local_ring,G_FILE_TEST_EXISTS)){
-			config->local_ring=PACKAGE_SOUND_DIR "/" LOCAL_RING;
-		}
-		if (strstr(config->local_ring,".wav")==NULL){
-			/* it currently uses old sound files, so replace them */
-			config->local_ring=PACKAGE_SOUND_DIR "/" LOCAL_RING;
-		}
-	}
-	config->remote_ring=gnome_config_get_string_with_default("sound/remote_ring",&def);
-	if (def) config->remote_ring=PACKAGE_SOUND_DIR "/" REMOTE_RING;
-	else {
-		if (!g_file_test(config->remote_ring,G_FILE_TEST_EXISTS)){
-			config->remote_ring=PACKAGE_SOUND_DIR "/" REMOTE_RING;
-		}
-		if (strstr(config->remote_ring,".wav")==NULL){
-			/* it currently uses old sound files, so replace them */
-			config->remote_ring=PACKAGE_SOUND_DIR "/" REMOTE_RING;
-		}
-	}
-	config->autokill=1;
+	if (def) tmpbuf="m";
+	linphone_core_set_sound_source(lc,tmpbuf[0]);
 	
-	sndcard=snd_card_manager_get_card(snd_card_manager,config->dev_id);
-	if (sndcard==NULL){
-		g_warning("Sound card with index %i does not exist. using 0 instead.",config->dev_id);
-		config->dev_id=0;
-		sndcard=snd_card_manager_get_card(snd_card_manager,config->dev_id);
-		
+	tmpbuf=gnome_config_get_string_with_default("sound/local_ring",&def);
+	if (def) tmpbuf=PACKAGE_SOUND_DIR "/" LOCAL_RING;
+	else {
+		if (!g_file_test(tmpbuf,G_FILE_TEST_EXISTS)){
+			tmpbuf=PACKAGE_SOUND_DIR "/" LOCAL_RING;
+		}
+		if (strstr(tmpbuf,".wav")==NULL){
+			/* it currently uses old sound files, so replace them */
+			tmpbuf=PACKAGE_SOUND_DIR "/" LOCAL_RING;
+		}
 	}
-	config->sndcard=sndcard;
-	g_return_if_fail(sndcard!=NULL);
-	snd_card_set_rec_source(sndcard,config->source);	
-	snd_card_set_level(sndcard,SND_CARD_LEVEL_OUTPUT,config->play_lev);
-	snd_card_set_level(sndcard,SND_CARD_LEVEL_INPUT,config->rec_lev);
-	tmp=test_audio_dev(config->dev_id);
-	if (tmp>0) config->latency=tmp;
-		
+	linphone_core_set_ring(lc,tmpbuf);
+	tmpbuf=gnome_config_get_string_with_default("sound/remote_ring",&def);
+	if (def) tmpbuf=PACKAGE_SOUND_DIR "/" REMOTE_RING_FR;
+	else {
+		if (!g_file_test(tmpbuf,G_FILE_TEST_EXISTS)){
+			tmpbuf=PACKAGE_SOUND_DIR "/" REMOTE_RING_FR;
+		}
+		if (strstr(tmpbuf,".wav")==NULL){
+			/* it currently uses old sound files, so replace them */
+			tmpbuf=PACKAGE_SOUND_DIR "/" REMOTE_RING_FR;
+		}
+	}
+	linphone_core_set_ringback(lc,0);
+	lc->sound_conf.autokill=1;
+	check_sound_device(lc);
 }
 
-void registrar_config_init(registrar_config_t *config)
+void registrar_config_read(LinphoneCore *lc)
 {
 	gboolean def;
-	config->use_registrar=gnome_config_get_int_with_default("sip/use_registrar",&def);
-	if (def) config->use_registrar=0;
-	config->as_proxy=gnome_config_get_int_with_default("sip/as_proxy",&def);
-	if (def) config->as_proxy=0;
-	config->expires=gnome_config_get_int_with_default("sip/expires",&def);
-	if (def) config->expires=15*60;
-	config->registrar=gnome_config_get_string_with_default("sip/registrar",&def);
-	if (def) config->registrar=NULL;
-	config->passwd=gnome_config_get_string_with_default("sip/passwd",&def);
-	if (def) config->passwd=NULL;
-	config->addr_of_rec=gnome_config_get_string_with_default("sip/addr_of_rec",&def);
-	if (def) config->addr_of_rec=NULL;
-	config->timer=g_timer_new();
-	g_timer_start(config->timer);
+	gchar *tmpstr,*tmpstr2;
+	gint tmp1,tmp2,tmp3;
+	
+	tmpstr=gnome_config_get_string_with_default("sip/registrar",&def);
+	if (def) tmpstr=NULL;
+	tmpstr2=gnome_config_get_string_with_default("sip/addr_of_rec",&def);
+	if (def) tmpstr2=NULL;
+	linphone_core_set_registrar_addr(lc,tmpstr,tmpstr2);
+	tmpstr=gnome_config_get_string_with_default("sip/passwd",&def);
+	if (def) tmpstr=NULL;
+	linphone_core_set_registrar_passwd(lc,tmpstr);
+	tmp1=gnome_config_get_int_with_default("sip/use_registrar",&def);
+	if (def) tmp1=0;
+	tmp2=gnome_config_get_int_with_default("sip/as_proxy",&def);
+	if (def) tmp2=0;
+	tmp3=gnome_config_get_int_with_default("sip/expires",&def);
+	if (def) tmp3=15*60;
+	linphone_core_set_registrar_usage(lc,tmp1,tmp2,tmp3);
+	
 }
 
-void sip_config_init(sip_config_t *config)
+void sip_config_read(LinphoneCore *lc)
 {
 	gboolean def;
-	config->username=gnome_config_get_string_with_default("sip/username",&def);
+	gchar *hostname,*username;
+	gint port;
+	username=gnome_config_get_string_with_default("sip/username",&def);
 	if (def) {
 		gchar *logname=getenv("LOGNAME");
 		if (logname!=NULL)
-			config->username=g_strdup(logname);
-		else config->username=g_strdup("toto");
+			username=g_strdup(logname);
+		else username=g_strdup("toto");
 	}
-	config->hostname=gnome_config_get_string_with_default("sip/hostname",&def);
+	hostname=gnome_config_get_string_with_default("sip/hostname",&def);
 	if (def) {
-		gchar *hostname=getenv("HOST");
-		if (hostname!=NULL)
-			config->hostname=g_strdup(hostname);
-		else config->hostname=g_strdup("unknown-host");
+		gchar *hostname2=getenv("HOST");
+		if (hostname2!=NULL)
+			hostname=g_strdup(hostname2);
+		else hostname=g_strdup("unknown-host");
 	}
-	config->sip_port=gnome_config_get_int_with_default("sip/sip_port",&def);
+	port=gnome_config_get_int_with_default("sip/sip_port",&def);
 	if (def){
-		config->sip_port=5060;
+		port=5060;
 	}
-	registrar_config_init(&config->reg_conf);
+	linphone_core_set_sip_port(lc,port);
+	linphone_core_set_primary_contact(lc,g_strdup_printf("sip:%s@%s",username,hostname));
+	registrar_config_read(lc);
 }
 
-void rtp_config_init(rtp_config_t *config)
+void rtp_config_read(LinphoneCore *lc)
 {
 	gboolean def;
-	config->audio_rtp_port=gnome_config_get_int_with_default("rtp/audio_rtp_port",&def);
-	if (def) config->audio_rtp_port=7078;
-	config->video_rtp_port=gnome_config_get_int_with_default("rtp/video_rtp_port",&def);
-	if (def) config->video_rtp_port=9078;
-	config->audio_jitt_comp=gnome_config_get_int_with_default("rtp/audio_jitt_comp",&def);
-	if (def) config->audio_jitt_comp=60;	
-	config->video_jitt_comp=gnome_config_get_int_with_default("rtp/video_jitt_comp",&def);
-	if (def) config->video_jitt_comp=60;	
+	gint port;
+	gint jitt_comp;
+	port=gnome_config_get_int_with_default("rtp/audio_rtp_port",&def);
+	if (def) port=7078;
+	linphone_core_set_audio_port(lc,port);
+	port=gnome_config_get_int_with_default("rtp/video_rtp_port",&def);
+	if (def) port=9078;
+	
+	jitt_comp=gnome_config_get_int_with_default("rtp/audio_jitt_comp",&def);
+	if (def) jitt_comp=60;
+	linphone_core_set_audio_jittcomp(lc,jitt_comp);		
+	jitt_comp=gnome_config_get_int_with_default("rtp/video_jitt_comp",&def);
+	if (def) jitt_comp=60;	
 	
 }
 
@@ -476,37 +500,42 @@ PayloadType * get_codec(char* type,int index){
 	return pt;
 }
 
-void codecs_config_init(codecs_config_t *config)
+void codecs_config_read(LinphoneCore *lc)
 {
 	int i;
 	PayloadType *pt;
-	config->audio_codecs=NULL;
-	config->video_codecs=NULL;
+	GList *audio_codecs=NULL;
+	GList *video_codecs=NULL;
 	for (i=0;;i++){
 		pt=get_codec("audio_codec",i);
 		if (pt==NULL) break;
-		config->audio_codecs=g_list_append(config->audio_codecs,(gpointer)pt);
+		audio_codecs=g_list_append(audio_codecs,(gpointer)pt);
 	}
 	for (i=0;;i++){
 		pt=get_codec("video_codec",i);
 		if (pt==NULL) break;
-		config->video_codecs=g_list_append(config->video_codecs,(gpointer)pt);
+		video_codecs=g_list_append(video_codecs,(gpointer)pt);
 	}
+	linphone_core_set_audio_codecs(lc,audio_codecs);
+	linphone_core_set_video_codecs(lc,video_codecs);
+	linphone_core_setup_local_rtp_profile(lc);
 }
 
-void video_config_init(video_config_t *config)
+void video_config_read(LinphoneCore *lc)
 {
 	gboolean def;
-	config->enabled=gnome_config_get_int_with_default("video/enabled",&def);
-	if (def) config->enabled=0;
-	config->show_local=gnome_config_get_int_with_default("video/show_local",&def);
-	if (def) config->show_local=0;
+	gint tmp;
+	tmp=gnome_config_get_int_with_default("video/enabled",&def);
+	if (def) tmp=0;
+	tmp=gnome_config_get_int_with_default("video/show_local",&def);
+	if (def) tmp=0;
 }
 
-void ui_config_init(ui_config_t *config)
+void ui_config_read(LinphoneCore *lc)
 {
 	int len,tmp;
 	char *ab_entry;
+	GList *address_list=NULL;
 	/* get address book entries*/
 	len= gnome_config_get_int("address_book/entry_count");
 	if (len!=-1)
@@ -514,10 +543,11 @@ void ui_config_init(ui_config_t *config)
 		for (tmp=0;tmp<len;tmp++)
 		{
 			ab_entry=g_strdup_printf("address_book/entry%i",tmp);
-			config->address_list=g_list_append(config->address_list,(gpointer)gnome_config_get_string(ab_entry));
+			address_list=g_list_append(address_list,(gpointer)gnome_config_get_string(ab_entry));
 			g_free(ab_entry);
 		}
 	}
+	linphone_core_set_address_list(lc,address_list);
 }
 
 void autoreplier_config_init(autoreplier_config_t *config)
@@ -548,10 +578,10 @@ linphone_core_init (LinphoneCore * lc,LinphoneCoreVTable *vtable, gchar *config_
 	ortp_init();
 	ortp_set_debug_file("oRTP",NULL);
 	rtp_profile_set_payload(&av_profile,115,&lpc1015);
-#ifndef __ARM__
+
 	rtp_profile_set_payload(&av_profile,110,&speex_nb);
 	rtp_profile_set_payload(&av_profile,111,&speex_wb);
-#endif
+
 	rtp_profile_set_payload(&av_profile,116,&truespeech);
 	rtp_profile_set_payload(&av_profile,101,&telephone_event);
 	ms_init();
@@ -572,19 +602,8 @@ linphone_core_init (LinphoneCore * lc,LinphoneCoreVTable *vtable, gchar *config_
 		gnome_config_push_prefix(g_strdup_printf("=%s=",config_path));
 	}else gnome_config_push_prefix("/linphone/");
 	
-	net_config_init(&lc->net_conf);
-	sound_config_init(&lc->sound_conf);
-	check_sound_device(lc);
-	sip_config_init(&lc->sip_conf);
-	rtp_config_init(&lc->rtp_conf);
-	codecs_config_init(&lc->codecs_conf);
-	video_config_init(&lc->video_conf);
-	autoreplier_config_init(&lc->autoreplier_conf);
-	ui_config_init(&lc->ui_conf);
-	
 	
 	lc->ua=osip_ua_new();
-	linphone_core_update_contact_info(lc);
   	err=osip_ua_signal_connect(lc->ua,"INVITE_ACCEPTED",invite_accepted_cb);
   	err=osip_ua_signal_connect(lc->ua,"BYE",bye_cb);
   	err=osip_ua_signal_connect(lc->ua,"FAILLURE",faillure_cb);
@@ -601,12 +620,19 @@ linphone_core_init (LinphoneCore * lc,LinphoneCoreVTable *vtable, gchar *config_
 	/* add the sdp handler to the ua */
 	osip_ua_add_body_handler(lc->ua,lc->sdph);
 	
-	linphone_core_set_registrar_usage(lc,lc->sip_conf.reg_conf.use_registrar,
-											lc->sip_conf.reg_conf.as_proxy);
-	linphone_core_setup_local_rtp_profile(lc);
-	linphone_core_set_connection_type(lc,lc->net_conf.con_type);
+	
+	sound_config_read(lc);
+	sip_config_read(lc);
+	net_config_read(lc);
+	rtp_config_read(lc);
+	codecs_config_read(lc);
+	video_config_read(lc);
+	//autoreplier_config_init(&lc->autoreplier_conf);
+	ui_config_read(lc);
+	
 	lc->in_main_thread=1;
 	lc->lock=g_mutex_new();
+	lc->vtable.display_status(lc,ready);
 }
 
 LinphoneCore *linphone_core_new(LinphoneCoreVTable *vtable,
@@ -668,14 +694,14 @@ from_t *linphone_core_get_primary_contact_parsed(LinphoneCore *lc)
 int linphone_core_set_audio_codecs(LinphoneCore *lc, GList *codecs)
 {
 	if (lc->codecs_conf.audio_codecs!=NULL) g_list_free(lc->codecs_conf.audio_codecs);
-	lc->codecs_conf.audio_codecs=g_list_copy(codecs);
+	lc->codecs_conf.audio_codecs=codecs;
 	return 0;
 }
 
 int linphone_core_set_video_codecs(LinphoneCore *lc, GList *codecs)
 {
 	if (lc->codecs_conf.video_codecs!=NULL) g_list_free(lc->codecs_conf.video_codecs);
-	lc->codecs_conf.video_codecs=g_list_copy(codecs);
+	lc->codecs_conf.video_codecs=codecs;
 	return 0;
 }
 
@@ -687,7 +713,7 @@ GList * linphone_core_get_address_list(LinphoneCore *lc)
 int linphone_core_set_address_list(LinphoneCore *lc, GList *addresses)
 {
 	if (lc->ui_conf.address_list!=NULL) g_list_free(lc->ui_conf.address_list);
-	lc->ui_conf.address_list=g_list_copy(addresses);
+	lc->ui_conf.address_list=addresses;
 	return 0;
 }
 
@@ -700,6 +726,7 @@ int linphone_core_set_registrar_addr(LinphoneCore *lc,char *url,char *addr_of_re
 {
 	url_t *surl=NULL,*surl2=NULL;
 	int err=0;
+	if (url==NULL || strlen(url)==0) return 0;
 	url_init(&surl);
 	err=url_parse(surl,url);
 	if (err<0){
@@ -773,16 +800,25 @@ int linphone_core_register(LinphoneCore *lc, char *url){
 	return 0;
 }
 
-void linphone_core_get_registrar_usage(LinphoneCore *lc,gboolean *reg, gboolean *proxy)
+void linphone_core_get_registrar_usage(LinphoneCore *lc,gboolean *reg, gboolean *proxy, gint *expires)
 {
 	if (reg!=NULL) *reg= lc->sip_conf.reg_conf.use_registrar;
 	if (proxy!=NULL) *proxy=lc->sip_conf.reg_conf.as_proxy;
+	if (expires!=NULL) *expires=lc->sip_conf.reg_conf.expires;
 }
 
 
 
-void linphone_core_set_registrar_usage(LinphoneCore *lc, gboolean use, gboolean outbound_proxy)
+void linphone_core_set_registrar_usage(LinphoneCore *lc, gboolean use, gboolean outbound_proxy,gint expires)
 {
+	if (lc->sip_conf.reg_conf.timer==NULL){
+		GTimer *timer;
+		timer=g_timer_new();
+		g_timer_start(timer);
+		lc->sip_conf.reg_conf.timer=timer;
+	}
+	if (expires<=0) lc->sip_conf.reg_conf.expires=15*60;
+	else lc->sip_conf.reg_conf.expires=expires;
 	if ((lc->sip_conf.reg_conf.registrar!=NULL) && (strlen(lc->sip_conf.reg_conf.registrar)!=0)){
 		if (use){
 			if (outbound_proxy)
@@ -882,9 +918,15 @@ void linphone_core_set_sip_port(LinphoneCore *lc,int port)
 	linphone_core_update_contact_info(lc);
 }
 
-void linphone_core_refresh(LinphoneCore *lc)
+void linphone_core_iterate(LinphoneCore *lc)
 {
 	check_for_registration(lc);
+  if (lc->preview_finished){
+    lc->preview_finished=0;
+    ring_stop(lc->ringstream);
+    lc->ringstream=NULL;
+    lc_callback_obj_invoke(&lc->preview_finished_cb,lc);
+  }
 }
 
 gboolean sipurl_check(gchar *url)
@@ -906,13 +948,28 @@ int linphone_core_invite(LinphoneCore *lc, char *url)
 {
 	gchar *barmsg;
 	gint err;
+	enum_lookup_res_t *enumres=NULL;
+	gchar *enum_domain;
 	
 	linphone_core_lock(lc);
-	if (!sipurl_check(url) ){
-		lc->vtable.display_warning(lc,_("Bad formuled sip address. A sip address looks like <sip:username@domainname>"));
-		linphone_core_unlock(lc);
-		return -1;
+	if (is_enum(url,&enum_domain)){
+		lc->vtable.display_status(lc,_("Looking for telephone number destination..."));
+		if (enum_lookup(enum_domain,&enumres)<0){
+			lc->vtable.display_status(lc,_("Could not resolve this number."));
+			g_free(enum_domain);
+			linphone_core_unlock(lc);
+			return -1;
+		}
+		g_free(enum_domain);
+		url=enumres->sip_address[0];
+	}else{
+		if (!sipurl_check(url) ){
+			lc->vtable.display_warning(lc,_("Bad formuled sip address. A sip address looks like <sip:username@domainname>"));
+			linphone_core_unlock(lc);
+			return -1;
+		}
 	}
+	
 	barmsg=g_malloc(strlen(url)+strlen(contacting)+2);
 	sprintf(barmsg,"%s %s",contacting,url);
 	lc->vtable.display_status(lc,barmsg);
@@ -923,6 +980,7 @@ int linphone_core_invite(LinphoneCore *lc, char *url)
 		g_warning("error in invite: %s (%i)\n",strerror(err),err);
 		lc->vtable.display_status(lc,ready);
 	}
+	if (enumres!=0) enum_lookup_res_free(enumres);
 	linphone_core_unlock(lc);
 	return 0;
 }
@@ -1036,10 +1094,24 @@ void linphone_core_set_presence_info(LinphoneCore *lc,gint minutes_away,
 													gchar *contact,
 													int presence_mode)
 {
+	int contactok=-1;
 	if (minutes_away>0) osip_ua_set_presence_delay(lc->ua,minutes_away*60);
 	if (contact!=NULL) {
-		osip_ua_set_presence_contact_url(lc->ua,contact);
+		from_t *url;
+		
+		from_init(&url);
+		contactok=from_parse(url,contact);
+		if (contactok>=0) {
+			osip_ua_set_presence_contact_url(lc->ua,contact);
+			g_message("contact url is correct.");
+		}
+		from_free(url);
+		sfree(url);
 	}
+	if (contactok<0 && (presence_mode==302 || presence_mode==380)){
+		return;
+	}
+	
 	osip_ua_set_presence_mode(lc->ua,presence_mode);
 }
 
@@ -1066,17 +1138,20 @@ void linphone_core_set_rec_level(LinphoneCore *lc, gint level)
 gint linphone_core_set_sound_device(LinphoneCore *lc, gint devid)
 {
 	SndCard *sndcard;
+	int tmp;
 	if (devid<0){
 		g_warning("Bad devid value: %i",devid);
-		return -1;
+		devid=0;
 	}
 	sndcard=snd_card_manager_get_card(snd_card_manager,devid);
 	if (sndcard==NULL){
 		g_warning("Sound card with index %i does not exist.");
-		return -1;
+		devid=0;
 	}
 	lc->sound_conf.dev_id=devid;
 	lc->sound_conf.sndcard=sndcard;
+	tmp=test_audio_dev(devid);
+	if (tmp>0) lc->sound_conf.latency=tmp;
 	return 0;
 }
 
@@ -1114,19 +1189,73 @@ void linphone_core_set_sound_source(LinphoneCore *lc, gchar source)
 	snd_card_set_rec_source(lc->sound_conf.sndcard,source);
 }
 
+void linphone_core_set_ring(LinphoneCore *lc,gchar *path){
+	if (lc->sound_conf.local_ring!=0){
+		g_free(lc->sound_conf.local_ring);
+	}
+	lc->sound_conf.local_ring=g_strdup(path);
+}
+gchar *linphone_core_get_ring(LinphoneCore *lc){
+	return lc->sound_conf.local_ring;
+}
+
+static void notify_end_of_ring(MSFilter *f,gint event, gpointer value,gpointer user_data){
+  LinphoneCore *lc=(LinphoneCore*)user_data;
+  lc->preview_finished=1;
+}
+
+gint linphone_core_preview_ring(LinphoneCore *lc, const gchar *ring,LinphoneCoreCbFunc func,gpointer userdata)
+{
+  if (lc->ringstream!=0){
+    g_warning("Cannot start ring now,there's already a ring being played");
+    return -1;
+  }
+  lc_callback_obj_init(&lc->preview_finished_cb,func,userdata);
+  lc->preview_finished=0;
+  if (try_open_dsp(lc)<0) return;
+  lc->ringstream=ring_start_with_cb((char*)ring,2,lc->sound_conf.sndcard,(MSFilterNotifyFunc)notify_end_of_ring,(gpointer)lc);
+  return 0;
+}
+
+
+void linphone_core_set_ringback(LinphoneCore *lc,RingBackType type){
+	switch(type){
+		case RINGBACK_TYPE_FR:
+			lc->sound_conf.remote_ring=PACKAGE_SOUND_DIR "/" REMOTE_RING_FR;
+		break;
+		case RINGBACK_TYPE_US:
+			lc->sound_conf.remote_ring=PACKAGE_SOUND_DIR "/" REMOTE_RING_US;
+		break;
+	}
+}
+RingBackType linphone_core_get_ringback(LinphoneCore *lc);
+
 void linphone_core_send_dtmf(LinphoneCore *lc,gchar dtmf)
 {
 	if (lc->audiostream!=NULL){
 		send_dtmf(lc->audiostream,dtmf);
 	}
 }
+void linphone_core_force_ip_address(LinphoneCore *lc, gchar *ipaddr)
+{
+	linphone_core_set_nat_address(lc,ipaddr,1);
+}
+
 
 void linphone_core_set_nat_address(LinphoneCore *lc, gchar *addr, gboolean use)
 {
 	gchar *tmp=NULL;
+	gint err;
+	struct addrinfo hints,*res;
+	
 	if (addr!=NULL && use){
-		if (inet_addr(addr)!=-1){
+		memset(&hints,0,sizeof(struct addrinfo));
+		hints.ai_family=PF_UNSPEC;
+		hints.ai_socktype = SOCK_DGRAM;
+		err=getaddrinfo(addr,NULL,&hints,&res);
+		if (err==0){
 			tmp=g_strdup(addr);
+			freeaddrinfo(res);
 		}else {
 			g_warning("Invalid nat address %s",addr);
 			return;
@@ -1150,8 +1279,8 @@ void linphone_core_set_nat_address(LinphoneCore *lc, gchar *addr, gboolean use)
 		lc->net_conf.use_nat=TRUE;
 	}
 	else {
-		linphone_core_update_contact_info(lc);
 		lc->net_conf.use_nat=FALSE;
+		linphone_core_update_contact_info(lc);
 	}
 }
 
@@ -1173,7 +1302,8 @@ void net_config_uninit(net_config_t *config)
 	gnome_config_set_int("net/con_type",config->con_type);
 	
 	gnome_config_set_int("net/use_nat",config->use_nat);
-	gnome_config_set_string("net/nat_address",config->nat_address);
+	if (config->nat_address!=NULL)
+		gnome_config_set_string("net/nat_address",config->nat_address);
 	
 }
 
@@ -1181,9 +1311,12 @@ void registrar_config_uninit(registrar_config_t *config)
 {
 	gnome_config_set_int("sip/use_registrar",config->use_registrar);
 	gnome_config_set_int("sip/as_proxy",config->as_proxy);
-	gnome_config_set_string("sip/registrar",config->registrar);
-	gnome_config_set_string("sip/passwd",config->passwd);
-	gnome_config_set_string("sip/addr_of_rec",config->addr_of_rec);
+	if (config->registrar!=NULL)
+		gnome_config_set_string("sip/registrar",config->registrar);
+	if (config->passwd!=NULL)
+		gnome_config_set_string("sip/passwd",config->passwd);
+	if (config->addr_of_rec!=NULL)
+		gnome_config_set_string("sip/addr_of_rec",config->addr_of_rec);
 	gnome_config_set_int("sip/expires",config->expires);
 	g_timer_destroy(config->timer);
 }
