@@ -19,39 +19,46 @@ Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 #ifndef LINPHONECORE_H
 #define LINPHONECORE_H
 
-
-#include <glib.h>
 #include <sndcard.h>
+#undef PACKAGE
+#undef VERSION
 #include <osipua.h>
 #undef PACKAGE
 #undef VERSION
-#include <ortp.h>
-#undef PACKAGE
-#undef VERSION
-
-
 
 #include <config.h>
-#include <libintl.h>
 
+#include <libintl.h>
 #ifndef _
 #define _(String) gettext(String)
 #endif
 
+#ifdef HAVE_GLIB
+#include <glib.h>
+#else
+#include <uglib.h>
+#endif
+
+
+
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-#define PAYLOAD_TYPE_ENABLED	(1<<7)
-#define PAYLOAD_TYPE_USABLE		(1<<8)
+#ifndef PayloadType_defined
+#define PayloadType_defined
+typedef struct _PayloadType PayloadType;
+#endif
 
-#define payload_type_enabled(pt) (((pt)->flags & PAYLOAD_TYPE_ENABLED)!=0)
-#define payload_type_usable(pt) ((pt)->flags & PAYLOAD_TYPE_USABLE)
-
-void payload_type_set_enable(PayloadType *pt,gint value);
-void payload_type_set_usable(PayloadType *pt,gint value);
-
-gchar *payload_type_get_description(PayloadType *pt);
+gboolean payload_type_enabled(struct _PayloadType *pt);
+gboolean payload_type_usable(struct _PayloadType *pt);
+void payload_type_set_enable(struct _PayloadType *pt,gint value);
+void payload_type_set_usable(struct _PayloadType *pt,gint value);
+gchar *payload_type_get_description(struct _PayloadType *pt);
+gint payload_type_get_bitrate(PayloadType *pt);
+const gchar *payload_type_get_mime(PayloadType *pt);
+gint payload_type_get_rate(PayloadType *pt);
 
 
 
@@ -175,7 +182,7 @@ typedef struct _LinphoneDialogParams
 {
 	StreamParams audio_params;
 	StreamParams video_params;
-	RtpProfile *profile;	/*points to the local_profile or to the remote "guessed" profile*/
+	struct _RtpProfile *profile;	/*points to the local_profile or to the remote "guessed" profile*/
 } LinphoneDialogParams;
 
 LinphoneDialogParams * linphone_dialog_params_new();
@@ -189,12 +196,15 @@ typedef void (*DisplayStatusCb)(struct _LinphoneCore *lc, char *message);
 typedef void (*DisplayMessageCb)(struct _LinphoneCore *lc, char *message);
 typedef void (*DisplayUrlCb)(struct _LinphoneCore *lc, char *message,char *url);
 typedef void (*DisplayQuestionCb)(struct _LinphoneCore *lc, char *message);
+typedef void (*LinphoneCoreCbFunc)(struct _LinphoneCore *lc,gpointer user_data);
+typedef void (*NotifyReceivedCb)(struct _LinphoneCore *lc, const char *url, const char *status);
 
 typedef struct _LinphoneVTable
 {
 	ShowInterfaceCb show;
 	InviteReceivedCb inv_recv;
 	ByeReceivedCb bye_recv;
+	NotifyReceivedCb notify_recv;
 	DisplayStatusCb display_status;
 	DisplayMessageCb display_message;
 	DisplayMessageCb display_warning;
@@ -202,6 +212,13 @@ typedef struct _LinphoneVTable
 	DisplayQuestionCb display_question;
 } LinphoneCoreVTable;
 
+typedef struct _LCCallbackObj
+{
+  LinphoneCoreCbFunc _func;
+  gpointer _user_data;
+}LCCallbackObj;
+
+
 typedef struct _LinphoneCore
 {
 	LinphoneCoreVTable vtable;
@@ -214,13 +231,15 @@ typedef struct _LinphoneCore
 	ui_config_t ui_conf;
 	autoreplier_config_t autoreplier_conf;
 	struct _RingStream *ringstream;
+	LCCallbackObj preview_finished_cb;
+	gboolean preview_finished;
 	OsipUA *ua;			/* the osip user agent*/
 	OsipDialog *call;   /* the current dialog, in the future it will be a list of dialog (conferencing)*/
 	GList *queued_calls;	/* used by the autoreplier */
 	BodyHandler *sdph;	/* the sdp handler */
 	struct _AudioStream *audiostream;  /**/
 	transaction_t *current_trn;	/* REVISIT LATER */
-	RtpProfile *local_profile;
+	struct _RtpProfile *local_profile;
 	int in_main_thread;
 	void *data;
 	GMutex *lock;
@@ -239,6 +258,8 @@ gchar *linphone_core_get_primary_contact(LinphoneCore *lc);
 
 from_t *linphone_core_get_primary_contact_parsed(LinphoneCore *lc);
 
+
+#ifdef LINPHONE_DEPRECATED
 GList * linphone_core_get_net_interfaces(LinphoneCore *lc);
 
 interface_t * linphone_core_get_selected_interface(LinphoneCore *lc);
@@ -249,6 +270,8 @@ void linphone_core_select_net_interface(LinphoneCore *lc, gint index);
 void linphone_core_select_net_interface(LinphoneCore *lc, gchar *ifname);
 #endif
 
+#endif
+
 void linphone_core_set_connection_type(LinphoneCore *lc, int type);
 
 int linphone_core_get_connection_type(LinphoneCore *lc);
@@ -289,9 +312,9 @@ int linphone_core_set_registrar_addr(LinphoneCore *lc, char *url,char*addr_of_re
 
 int linphone_core_register(LinphoneCore *lc, char *url);
 
-void linphone_core_get_registrar_usage(LinphoneCore *lc, gboolean *enabled, gboolean *as_proxy);
+void linphone_core_get_registrar_usage(LinphoneCore *lc, gboolean *enabled, gboolean *as_proxy, gint*expires);
 
-void linphone_core_set_registrar_usage(LinphoneCore *lc, gboolean enabled, gboolean as_proxy);
+void linphone_core_set_registrar_usage(LinphoneCore *lc, gboolean enabled, gboolean as_proxy, gint expires);
 
 int linphone_core_get_audio_jittcomp(LinphoneCore *lc);
 
@@ -309,8 +332,10 @@ void linphone_core_set_nat_address(LinphoneCore *lc, gchar *addr, gboolean use);
 
 gchar *linphone_core_get_nat_address(LinphoneCore *lc, gboolean *use);
 
+void linphone_core_force_ip_address(LinphoneCore *lc, gchar *ipaddr);
+
 /* function to be periodically called in a main loop */
-void linphone_core_refresh(LinphoneCore *lc);
+void linphone_core_iterate(LinphoneCore *lc);
 
 /* sound functions */
 SndCardManager * linphone_core_get_card_manager(LinphoneCore *lc);
@@ -323,6 +348,12 @@ gint linphone_core_set_sound_device(LinphoneCore *lc, gint devid);
 gint linphone_core_set_sound_device_from_name(LinphoneCore *lc,gchar *name);
 gchar linphone_core_get_sound_source(LinphoneCore *lc);
 void linphone_core_set_sound_source(LinphoneCore *lc, gchar source);
+void linphone_core_set_ring(LinphoneCore *lc,gchar *path);
+gchar *linphone_core_get_ring(LinphoneCore *lc);
+gint linphone_core_preview_ring(LinphoneCore *lc, const gchar *ring,LinphoneCoreCbFunc func,gpointer userdata);
+typedef enum {RINGBACK_TYPE_FR,RINGBACK_TYPE_US} RingBackType;
+void linphone_core_set_ringback(LinphoneCore *lc,RingBackType type);
+RingBackType linphone_core_get_ringback(LinphoneCore *lc);
 
 void linphone_core_set_presence_info(LinphoneCore *lc,gint minutes_away,
 													gchar *contact,
