@@ -14,6 +14,7 @@
 #include "propertybox.h"
 #include "presence.h"
 #include "addressbook.h"
+#include "friends.h"
 
 typedef struct _LinphoneMainWindow
 {
@@ -23,6 +24,7 @@ typedef struct _LinphoneMainWindow
 	GtkWidget *optioncontrols;
 	GtkWidget *dtmfentry;
 	PresenceBox presencebox;
+	FriendList friendlist;
 }LinphoneMainWindow;
 
 typedef struct _LinphoneGnomeUI
