@@ -270,7 +270,7 @@ void sip_section_enable_registrar(SipSection *sec,LinphoneCore *lc, gboolean sta
 	
  	if (state) {
 		gboolean flag;
-		linphone_core_get_registrar_usage(lc,NULL,&flag);
+		linphone_core_get_registrar_usage(lc,NULL,&flag,NULL);
  		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(sec->obproxy_button),flag);
  	}
  	else {
@@ -303,7 +303,7 @@ void sip_section_fill(SipSection *sec, LinphoneCore *lc)
 	tmp=linphone_core_get_addr_of_rec(lc);
 	/* set address of record */
 	if (tmp==NULL) gtk_entry_set_text(GTK_ENTRY(sec->addr_of_rec),tmp);
-	linphone_core_get_registrar_usage(lc,&flag,NULL);
+	linphone_core_get_registrar_usage(lc,&flag,NULL,NULL);
 	if (flag)
 	{
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(sec->registrar_toggle),1);
@@ -351,7 +351,7 @@ void sip_section_apply(SipSection *sec, LinphoneCore *lc)
 		g_free(addr_of_rec);
 		g_free(passwd);
 	}
-	linphone_core_set_registrar_usage(lc,state,as_proxy);
+	linphone_core_set_registrar_usage(lc,state,as_proxy,0);
 }
 
 void codec_section_init(CodecSection *sec, GtkWidget *prop)
@@ -365,7 +365,7 @@ void codec_selection_changed_cb(GtkTreeSelection *selection, gpointer data)
 {
 	GtkTreeIter iter;
     GtkTreeModel *model;
-    PayloadType *pt=NULL;
+    struct _PayloadType *pt=NULL;
 	LinphonePropertyBox *prop=get_property_box();
 	
 	if (gtk_tree_selection_get_selected (selection, &model, &iter))	
@@ -394,16 +394,16 @@ void codec_list_build(GtkTreeView *listview,GList *codeclist)
 		gchar *status;
 		gchar *bitrate,*rate;
 		gchar *color;
-		PayloadType *pt=(PayloadType *)elem->data;
+		struct _PayloadType *pt=(struct _PayloadType *)elem->data;
 		if (payload_type_enabled(pt)) status=_("Enabled");
 		else status=_("Disabled");
 		if (payload_type_usable(pt)) color="blue";
 		else color="red";
 		/* get an iterator */
 		gtk_list_store_append(store,&iter);
-		bitrate=g_strdup_printf("%g",((double)pt->normal_bitrate)/1000.0);
-		rate=g_strdup_printf("%i",pt->clock_rate);
-		gtk_list_store_set(store,&iter,	CODEC_NAME,pt->mime_type,
+		bitrate=g_strdup_printf("%g",((double)payload_type_get_bitrate(pt))/1000.0);
+		rate=g_strdup_printf("%i",payload_type_get_rate(pt));
+		gtk_list_store_set(store,&iter,	CODEC_NAME,payload_type_get_mime(pt),
 										CODEC_RATE,rate,
 										CODEC_BITRATE,bitrate,
 										CODEC_STATUS,status,
@@ -461,6 +461,7 @@ void codec_section_fill(CodecSection *sec, LinphoneCore *lc)
 	/* display prefered codecs*/
 	GList *audio_codecs=linphone_core_get_audio_codecs(lc);
 	codec_list_build(GTK_TREE_VIEW(sec->au_codec_list),audio_codecs);
+	g_list_free(audio_codecs);
 	menu=gtk_option_menu_get_menu(GTK_OPTION_MENU(optionmenu));
 	menushell=GTK_MENU_SHELL(menu);
 	elem=item_list=menushell->children;
@@ -494,7 +495,7 @@ void codec_section_fill(CodecSection *sec, LinphoneCore *lc)
 void codec_section_apply(CodecSection *sec, LinphoneCore *lc)
 {
 	GtkTreeIter iter;
-	PayloadType *pt;
+	struct _PayloadType *pt;
 	GList *codeclist=NULL;
 	gchar *status;
 	GtkTreeModel *model=gtk_tree_view_get_model(GTK_TREE_VIEW(sec->au_codec_list));
@@ -515,6 +516,7 @@ void sound_section_init(SoundSection *sec, GtkWidget *prop)
 	sec->card_entry=lookup_widget(prop,"card_entry");
 	sec->card_combo=lookup_widget(prop,"card_combo");
 	sec->source_entry=lookup_widget(prop,"rec_source");
+	sec->ringfileentry=lookup_widget(prop,"ringfileentry");
 }
 
 void sound_section_fill(SoundSection *sec, LinphoneCore *lc)
@@ -527,7 +529,7 @@ void sound_section_fill(SoundSection *sec, LinphoneCore *lc)
 	int i;
 	/* select used sound drivers*/
 
-  	devid=linphone_core_get_sound_device(lc);
+	devid=linphone_core_get_sound_device(lc);
 	cardmanager=linphone_core_get_card_manager(lc);
 	
 	for (i=0;(card=snd_card_manager_get_card(cardmanager,i))!=0;i++){
@@ -538,8 +540,8 @@ void sound_section_fill(SoundSection *sec, LinphoneCore *lc)
 	g_return_if_fail(card!=NULL);
 	gtk_entry_set_text (GTK_ENTRY (sec->card_entry),snd_card_get_identifier(card));
 	
-  	/* select audio source*/
-  	switch(linphone_core_get_sound_source(lc))
+	/* select audio source*/
+	switch(linphone_core_get_sound_source(lc))
 	{
 		case 'm':
 			gtk_entry_set_text (GTK_ENTRY (sec->source_entry), _("micro"));
@@ -549,26 +551,74 @@ void sound_section_fill(SoundSection *sec, LinphoneCore *lc)
   			break;
 		default:
 			g_warning("Invalid source !");
-  	}
+  }
+  {
+		gchar *ringfile=linphone_core_get_ring(lc);
+		gnome_file_entry_set_filename(GNOME_FILE_ENTRY(sec->ringfileentry),ringfile);
+		
+  }
 }
 
+void
+on_card_changed                        (GtkEditable     *editable,
+                                        gpointer         user_data)
+{
+	gchar *tmp;
+	
+	/* get sound device*/
+	tmp=gtk_editable_get_chars(editable,0,-1);
+	linphone_core_set_sound_device_from_name(get_core(),tmp);
+	g_free(tmp);
+}
+
+
 void sound_section_apply(SoundSection *sec, LinphoneCore *lc)
 {
 	gchar *tmp;
 	int devid;
-	/* get sound device*/
 	
+	/* get sound device*/
 	tmp=gtk_editable_get_chars(GTK_EDITABLE(sec->card_entry),0,-1);
 	linphone_core_set_sound_device_from_name(lc,tmp);
 	g_free(tmp);
 	
-  	/* get audio source*/
-  	tmp=gtk_editable_get_chars(GTK_EDITABLE(sec->source_entry),0,-1);
+	/* get audio source*/
+	tmp=gtk_editable_get_chars(GTK_EDITABLE(sec->source_entry),0,-1);
 	if (strcmp(tmp,_("micro"))==0) linphone_core_set_sound_source(lc,'m');
 	else if (strcmp(tmp,_("line"))==0) linphone_core_set_sound_source(lc,'l');
 	g_free(tmp);
+	
+	/* get ring path */
+	tmp=gtk_editable_get_chars(GTK_EDITABLE(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(sec->ringfileentry))),0,-1);
+	linphone_core_set_ring(lc,tmp);
+	g_free(tmp);
+	
 }
 
+void ring_finished(LinphoneCore *lc,gpointer user_data)
+{
+  GtkWidget *button=(GtkWidget*)user_data;
+  LinphonePropertyBox *prop=get_property_box();
+  if (prop->prop==NULL) return; /* the box has been closed before the end of the preview */
+  gtk_widget_set_sensitive(button,1);
+}
+
+void
+on_ringpreview_clicked                 (GtkButton       *button,
+                                        gpointer         user_data)
+{
+  int err;
+  gchar *tmp;
+  LinphonePropertyBox *prop=get_property_box();
+	LinphoneCore *lc=get_core();
+  /* get ring path */
+	tmp=gtk_editable_get_chars(GTK_EDITABLE(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(prop->sound.ringfileentry))),0,-1);
+	err=linphone_core_preview_ring(lc,tmp,ring_finished,(gpointer)button);
+  if (err==0) gtk_widget_set_sensitive(GTK_WIDGET(button),0);
+	g_free(tmp);
+}
+
+
 void linphone_property_box_fill(LinphonePropertyBox * box, LinphoneCore *lp)
 {
 	net_section_fill(&box->net,lp);
@@ -659,7 +709,7 @@ void codec_row_move(GtkTreeView *listview, int action)
     GtkTreeModel *model;
 	GtkTreeSelection *selection;
 	GtkTreePath *treepath;
-    PayloadType *codec=NULL;
+    struct _PayloadType *codec=NULL;
 	gchar *name=NULL;
 	gchar *rate=NULL;
 	gchar *bitrate=NULL;
@@ -758,7 +808,7 @@ void codec_list_update(GtkTreeView *listview,double bandwidth)
 {
 	GtkTreeModel *model;
 	GtkTreeIter iter;
-	PayloadType *pt;
+	struct _PayloadType *pt;
 	gchar *color;
 	model=gtk_tree_view_get_model(listview);
 	
@@ -850,6 +900,7 @@ on_registrar_addr_changed              (GtkEditable     *editable,
 	hostname=url1->host;
 	addrrec=gtk_editable_get_chars(GTK_EDITABLE(address_of_record),0,-1);
 	url_init(&url2);
+
 	if ((addrrec!=NULL) && (strlen(addrrec)!=0)){
 		err=url_parse(url2,addrrec);
 		//printf("address of record parsed with err=%i\n",err);
@@ -904,12 +955,6 @@ on_address_of_record_changed           (GtkEditable     *editable,
     property_box_changed();
 }
 
-void
-on_card_changed                        (GtkEditable     *editable,
-                                        gpointer         user_data)
-{
-	property_box_changed();
-}
 
 void
 on_audio_jittcomp_value_changed        (GtkRange        *range,
@@ -917,3 +962,25 @@ on_audio_jittcomp_value_changed        (GtkRange        *range,
 {
 	property_box_changed();
 }
+
+void
+on_ringfileentry_changed               (GtkEditable     *editable,
+                                        gpointer         user_data)
+{
+	property_box_changed();
+}
+
+void linphone_property_box_uninit(LinphonePropertyBox *box)
+{
+  memset(box,0, sizeof(LinphonePropertyBox));
+}
+
+
+
+gboolean
+on_property_box_closed                 (GnomeDialog     *gnomedialog,
+                                        gpointer         user_data)
+{
+  linphone_property_box_uninit(get_property_box());
+  return FALSE;
+}
