@@ -75,6 +75,7 @@ int alsa_set_params(AlsaCard *obj, int rw, int bits, int stereo, int rate)
 	/* Set sample rate. If the exact rate is not supported */
     /* by the hardware, use nearest possible rate.         */ 
 	exact_value=rate;
+	dir=0;
     if ((err=snd_pcm_hw_params_set_rate_near(pcm_handle, hwparams, &exact_value, &dir))<0){
 		g_warning("alsa_set_params: Error setting rate to %i:%s",rate,snd_strerror(err));
  		return -1;
@@ -203,6 +204,10 @@ int alsa_card_open_w(AlsaCard *obj,int bits,int stereo,int rate)
 }
 
 
+void alsa_card_set_blocking_mode(AlsaCard *obj, gboolean yesno){
+	if (obj->read_handle!=NULL) snd_pcm_nonblock(obj->read_handle,!yesno);
+	if (obj->write_handle!=NULL) snd_pcm_nonblock(obj->write_handle,!yesno);
+}
 
 void alsa_card_close_r(AlsaCard *obj)
 {
@@ -307,7 +312,7 @@ int __alsa_card_write(AlsaCard *obj,char *buf,int size)
 	sigprocmask(SIG_BLOCK,&set,NULL);
 	if ((err=snd_pcm_writei(obj->write_handle,buf,size/obj->frame_size))<0){
 		if (err!=-EPIPE){
-			g_warning("alsa_card_read: snd_pcm_writei() failed:%s.",snd_strerror(err));
+			g_warning("alsa_card_write: snd_pcm_writei() failed:%s.",snd_strerror(err));
 		}
 		snd_pcm_prepare(obj->write_handle);
 		err=snd_pcm_writei(obj->write_handle,buf,size/obj->frame_size);
@@ -371,7 +376,7 @@ void alsa_mixer_close(AlsaCard *obj){
 	obj->mixer=NULL;
 }
 
-typedef enum {CAPTURE, PLAYBACK, CAPTURE_SWITCH} MixerAction;
+typedef enum {CAPTURE, PLAYBACK, CAPTURE_SWITCH, PLAYBACK_SWITCH} MixerAction;
 
 static gint get_mixer_element(snd_mixer_t *mixer,const char *name, MixerAction action){
 	long value=0;
@@ -444,11 +449,18 @@ static void set_mixer_element(snd_mixer_t *mixer,const char *name, gint level,Mi
 					//g_message("Succesfully set capture switch for %s.",elemname);
 				}
 				break;
+				case PLAYBACK_SWITCH:
+				if (snd_mixer_selem_has_playback_switch(elem)){
+					snd_mixer_selem_set_playback_switch_all(elem,level);
+					//g_message("Succesfully set capture switch for %s.",elemname);
+				}
+				break;
+
 			}
 		}
 		elem=snd_mixer_elem_next(elem);
 	}
-	
+
 	return ;
 }
 
@@ -505,9 +517,11 @@ void alsa_card_set_source(AlsaCard *obj,int source)
 	switch (source){
 		case 'm':
 			set_mixer_element(mixer,"Mic",1,CAPTURE_SWITCH);
+			set_mixer_element(mixer,"Capture",1,CAPTURE_SWITCH);
 			break;
 		case 'l':
 			set_mixer_element(mixer,"Line",1,CAPTURE_SWITCH);
+			set_mixer_element(mixer,"Capture",1,CAPTURE_SWITCH);
 			break;
 	}
 }
@@ -553,6 +567,7 @@ SndCard * alsa_card_new(gint devid)
 	base->_open_r=(SndCardOpenFunc)alsa_card_open_r;
 	base->_open_w=(SndCardOpenFunc)alsa_card_open_w;
 	base->_can_read=(SndCardPollFunc)alsa_card_can_read;
+	base->_set_blocking_mode=(SndCardSetBlockingModeFunc)alsa_card_set_blocking_mode;
 	base->_read=(SndCardIOFunc)alsa_card_read;
 	base->_write=(SndCardIOFunc)alsa_card_write;
 	base->_close_r=(SndCardCloseFunc)alsa_card_close_r;
