@@ -100,7 +100,7 @@ void rtp_parse(RtpSession *session,mblk_t *mp)
 	
 	rtp_putq(session->rtp.rq,mp);
 	/* make some checks: q size must not exceed RtpStream::max_rq_size */
-	while (session->rtp.rq->q_count > session->rtp.max_rq_size)
+	while (session->rtp.rq->q_mcount > session->rtp.max_rq_size)
 	{
 		/* remove the oldest mblk_t */
 		mp=getq(session->rtp.rq);
