@@ -75,6 +75,7 @@ rtp_session_init (RtpSession * session, gint mode)
 	rtp_signal_table_init (&session->on_payload_type_changed, session);
 	rtp_signal_table_init (&session->on_telephone_event, session);
 	rtp_signal_table_init (&session->on_telephone_event_packet, session);
+	rtp_signal_table_init (&session->on_timestamp_jump,session);
 #ifdef BUILD_SCHEDULER
 	session->rtp.wait_for_packet_to_be_sent_mutex = g_mutex_new ();
 	session->rtp.wait_for_packet_to_be_sent_cond = g_cond_new ();
@@ -329,7 +330,7 @@ rtp_session_set_local_addr (RtpSession * session, gchar * addr, gint port)
 		session->rtp.socket = socket(res->ai_family, res->ai_socktype, 0);
 		if (session->rtp.socket < 0)
 		  continue;
-
+                
 		/* set non blocking mode */
 		set_non_blocking_socket (session);
 		memcpy(&session->rtp.loc_addr, res->ai_addr, res->ai_addrlen);
@@ -528,7 +529,7 @@ rtp_session_set_remote_addr (RtpSession * session, gchar * addr, gint port)
 #else
 			err = rtp_session_set_local_addr (session, "0.0.0.0", localport);
 #endif
-							  
+
 			if (err == 0)
 				break;
 		}
@@ -548,9 +549,12 @@ rtp_session_set_remote_addr (RtpSession * session, gchar * addr, gint port)
 		g_warning ("Error in socket address: %s", gai_strerror(err));
 		return err;
 	}
-
+	
 	for (res = res0; res; res = res->ai_next) {
-		err = connect (session->rtp.socket, res->ai_addr, res->ai_addrlen);
+		/*err = connect (session->rtp.socket, res->ai_addr, res->ai_addrlen);
+		*/
+		/*don't connect: after you are no more able to use recvfrom() and sendto() */
+		err=0;
 		if (err == 0) {
 		  memcpy( &session->rtp.rem_addr, res->ai_addr, res->ai_addrlen);
 		  break;
@@ -573,7 +577,9 @@ rtp_session_set_remote_addr (RtpSession * session, gchar * addr, gint port)
 		return err;
 	}
 	for (res = res0; res; res = res->ai_next) {
-		err = connect (session->rtcp.socket, res->ai_addr, res->ai_addrlen);
+		/*err = connect (session->rtcp.socket, res->ai_addr, res->ai_addrlen);*/
+		/*don't connect: after you are no more able to use recvfrom() and sendto() */
+		err=0;
 		if (err == 0) {
 		  memcpy( &session->rtcp.rem_addr, res->ai_addr, res->ai_addrlen);
 		  break;
@@ -632,8 +638,25 @@ void rtp_session_set_sockets(RtpSession *session, gint rtpfd, gint rtcpfd)
 {
 	session->rtp.socket=rtpfd;
 	session->rtcp.socket=rtcpfd;
+	session->flags|=RTP_SESSION_USING_EXT_SOCKETS;
 }
 
+/**
+ *rtp_session_set_seq_number
+ *@session:		a rtp session freshly created.
+ *@addr:			a 16 bit unsigned number.
+ *
+ * sets the initial sequence number of a sending session.
+ *
+**/
+void rtp_session_set_seq_number(RtpSession *session, guint16 seq){
+	session->rtp.snd_seq=seq;
+}
+
+
+guint16 rtp_session_get_seq_number(RtpSession *session){
+	return session->rtp.snd_seq;
+}
 
 #ifdef TARGET_IS_HPUXKERNEL
 #ifdef WORDS_BIGENDIAN
@@ -689,14 +712,13 @@ rtp_send (RtpSession * session, mblk_t * m)
 	
 	for (i = 0; i < hdr->cc; i++)
 		hdr->csrc[i] = htonl (hdr->csrc[i]);
-	error = sendto (session->rtp.socket, m->b_rptr,
-			(m->b_wptr - m->b_rptr), 0,
-			(struct sockaddr *) &session->rtp.rem_addr,
-#ifdef INET6
-			sizeof (session->rtp.rem_addr));
-#else
-			sizeof (struct sockaddr_in));
-#endif
+	if (session->flags & RTP_SESSION_USING_EXT_SOCKETS){
+		error=send(session->rtp.socket, m->b_rptr, (m->b_wptr - m->b_rptr),0);
+	}else error = sendto (session->rtp.socket, m->b_rptr,
+		(m->b_wptr - m->b_rptr), 0,
+		(struct sockaddr *) &session->rtp.rem_addr,
+		sizeof (session->rtp.rem_addr));
+
 	if (error < 0)
 		g_warning ("Error sending rtp packet: %s.", getSocketError());
 	freemsg (m);
@@ -767,6 +789,11 @@ rtp_session_set_payload_type (RtpSession * session, int paytype)
 	return 0;
 }
 
+int rtp_session_get_payload_type(RtpSession *session){
+	return session->payload_type;
+}
+
+
 /**
  *rtp_session_set_payload_type_with_string:
  *@session: a rtp session
@@ -872,7 +899,7 @@ rtp_session_sendm_with_ts (RtpSession * session, mblk_t *mp, guint32 timestamp)
 			sched = ortp_get_scheduler ();
 			
 			session->rtp.snd_time_offset = sched->time_;
-			//g_message("setting snd_time_offset=%i",session->rtp.snd_time_offset);
+			//ortp_debug("setting snd_time_offset=%i",session->rtp.snd_time_offset);
 		}
 #endif
 		rtp_session_unset_flag (session,RTP_SESSION_SEND_NOT_STARTED);
@@ -924,12 +951,12 @@ rtp_session_sendm_with_ts (RtpSession * session, mblk_t *mp, guint32 timestamp)
 				     timestamp -
 				     session->rtp.snd_ts_offset) +
 					session->rtp.snd_time_offset;
-		//g_message("rtp_session_send_with_ts: packet_time=%i time=%i",packet_time,sched->time_);
+		//ortp_debug("rtp_session_send_with_ts: packet_time=%i time=%i",packet_time,sched->time_);
 		if (TIME_IS_STRICTLY_NEWER_THAN (packet_time, sched->time_))
 		{
 			if (session->flags & RTP_SESSION_BLOCKING_MODE)
 			{
-				//g_message("waiting packet to be sent");
+				//ortp_debug("waiting packet to be sent");
 				g_mutex_lock (session->rtp.
 				      wait_for_packet_to_be_sent_mutex);
 				g_cond_wait (session->rtp.
@@ -969,7 +996,6 @@ rtp_session_send_with_ts (RtpSession * session, gchar * buffer, gint len,
 			  guint32 userts)
 {
 	mblk_t *m;
-	gint error = 0;
 	gint msgsize;
 
 	/* allocate a mblk_t, set the haeder. Perhaps if len>MTU, we should allocate a new
@@ -989,13 +1015,13 @@ rtp_session_send_with_ts (RtpSession * session, gchar * buffer, gint len,
 
 #ifdef TARGET_IS_HPUXKERNEL
 static gint
-rtp_stack_recv (RtpSession * session)
+rtp_recv (RtpSession * session)
 {
 	return EOPNOTSUPP;
 }
 #else
 static gint
-rtp_stack_recv (RtpSession * session)
+rtp_recv (RtpSession * session)
 {
 	gint error;
 	struct sockaddr_in remaddr;
@@ -1005,8 +1031,10 @@ rtp_stack_recv (RtpSession * session)
 	fd_set fdset;
 	struct timeval timeout = { 0, 0 };
 
-	g_return_val_if_fail(session->rtp.socket>0,-1);
+	if (session->rtp.socket<1) return -1;  /*session has no sockets for the moment*/
 	FD_ZERO (&fdset);
+	if (!session)
+		printf("Session null");
 	FD_SET (session->rtp.socket, &fdset);
 
 	while (1)
@@ -1017,7 +1045,9 @@ rtp_stack_recv (RtpSession * session)
 		if ((error == 1) && (FD_ISSET (session->rtp.socket, &fdset)))	/* something to read */
 		{
 			mp = allocb (session->max_buf_size, 0);
-			error = recvfrom (session->rtp.socket, mp->b_wptr,
+			if (session->flags & RTP_SESSION_USING_EXT_SOCKETS){
+				error=recv(session->rtp.socket,mp->b_wptr,session->max_buf_size,0);
+			}else error = recvfrom (session->rtp.socket, mp->b_wptr,
 					  session->max_buf_size, 0,
 					  (struct sockaddr *) &remaddr,
 					  &addrlen);
@@ -1027,7 +1057,7 @@ rtp_stack_recv (RtpSession * session)
 
 				p = g_realloc (mp->b_wptr, error);
 				if (p != mp->b_wptr)
-					g_message
+					ortp_debug
 						("The recv area has moved during reallocation.");
 				mp->b_datap->db_base = mp->b_rptr =
 					mp->b_wptr = p;
@@ -1096,7 +1126,7 @@ rtp_session_recvm_with_ts (RtpSession * session, guint32 user_ts)
 		{
 			sched = ortp_get_scheduler ();
 			session->rtp.rcv_time_offset = sched->time_;
-			//g_message("setting snd_time_offset=%i",session->rtp.snd_time_offset);
+			//ortp_debug("setting snd_time_offset=%i",session->rtp.snd_time_offset);
 		}
 #endif
 		rtp_session_unset_flag (session,RTP_SESSION_RECV_NOT_STARTED);
@@ -1107,7 +1137,7 @@ rtp_session_recvm_with_ts (RtpSession * session, guint32 user_ts)
 #else
 	if (!(session->flags & RTP_SESSION_SCHEDULED))	/* if not scheduled */
 	{
-		rtp_stack_recv (session);
+		rtp_recv (session);
 	}
 #endif
 	/* then now try to return a packet, if possible */
@@ -1121,7 +1151,7 @@ rtp_session_recvm_with_ts (RtpSession * session, guint32 user_ts)
 		queue_t *q = session->rtp.rq;
 		if (q->q_last == NULL)
 		{
-			g_message ("Queue is empty.");
+			ortp_debug ("Queue is empty.");
 			goto end;
 		}
 		oldest = (rtp_header_t *) q->q_first->b_rptr;
@@ -1129,7 +1159,7 @@ rtp_session_recvm_with_ts (RtpSession * session, guint32 user_ts)
 		if ((guint32) (newest->timestamp - oldest->timestamp) <
 		    session->rtp.jitt_comp_time)
 		{
-			g_message ("Not enough packet bufferised.");
+			ortp_debug ("Not enough packet bufferised.");
 			goto end;
 		}
 		/* if enough packet queued continue but delete the starting flag */
@@ -1144,7 +1174,7 @@ rtp_session_recvm_with_ts (RtpSession * session, guint32 user_ts)
 		session->rtp.rcv_diff_ts = rtp->timestamp - user_ts;
 		session->rtp.rcv_last_ret_ts = user_ts;	/* just to have an init value */
 		session->ssrc = rtp->ssrc;
-		g_message ("Returning FIRST mp with ts=%i", rtp->timestamp);
+		ortp_debug ("Returning FIRST mp with ts=%i", rtp->timestamp);
 
 		goto end;
 	}
@@ -1166,7 +1196,7 @@ rtp_session_recvm_with_ts (RtpSession * session, guint32 user_ts)
 		ortp_global_stats.recv += msgsize;
 		session->stats.recv += msgsize;
 		rtp = (rtp_header_t *) mp->b_rptr;
-		g_message ("Returning mp with ts=%i", rtp->timestamp);
+		ortp_debug("Returning mp with ts=%i", rtp->timestamp);
 		/* check for payload type changes */
 		if (session->payload_type != rtp->paytype)
 		{
@@ -1184,17 +1214,23 @@ rtp_session_recvm_with_ts (RtpSession * session, guint32 user_ts)
 				freemsg(mp);
 				mp=NULL;
 			}else{
-				g_message ("rtp_parse: payload type changed to %i !",
-				   rtp->paytype);
-				session->payload_type = rtp->paytype;
-				rtp_signal_table_emit (&session->on_payload_type_changed);
+				/* check if we support this payload type */
+				PayloadType *pt=rtp_profile_get_payload(session->profile,rtp->paytype);
+				if (pt!=0){
+					ortp_debug ("rtp_parse: payload type changed to %i !",
+						 rtp->paytype);
+					session->payload_type = rtp->paytype;
+					rtp_signal_table_emit (&session->on_payload_type_changed);
+				}else{
+					g_warning("Receiving packet with unknown payload type %i.",rtp->paytype);
+				}
 			}
 		}
 		
 	}
 	else
 	{
-		g_message ("No mp for timestamp queried");
+		ortp_debug ("No mp for timestamp queried");
 		session->stats.unavaillable++;
 		ortp_global_stats.unavaillable++;
 	}
@@ -1212,7 +1248,7 @@ rtp_session_recvm_with_ts (RtpSession * session, guint32 user_ts)
 				     user_ts -
 				     session->rtp.rcv_query_ts_offset) +
 			session->rtp.rcv_time_offset;
-		//g_message ("rtp_session_recvm_with_ts: packet_time=%i, time=%i",packet_time, sched->time_);
+		//ortp_debug ("rtp_session_recvm_with_ts: packet_time=%i, time=%i",packet_time, sched->time_);
 		if (TIME_IS_STRICTLY_NEWER_THAN (packet_time, sched->time_))
 		{
 			if (session->flags & RTP_SESSION_BLOCKING_MODE)
@@ -1324,7 +1360,7 @@ gint rtp_session_recv_with_ts (RtpSession * session, gchar * buffer,
 	}
 	if (!(session->flags & RTP_SESSION_RECV_SYNC))
 	{
-		//g_message("time=%i   rcv_last_ret_ts=%i",time,session->rtp.rcv_last_ret_ts);
+		//ortp_debug("time=%i   rcv_last_ret_ts=%i",time,session->rtp.rcv_last_ret_ts);
 		if (RTP_TIMESTAMP_IS_STRICTLY_NEWER_THAN
 		    (time, session->rtp.rcv_last_ret_ts))
 		{
@@ -1339,7 +1375,7 @@ gint rtp_session_recv_with_ts (RtpSession * session, gchar * buffer,
 			ts_int = (guint32) (((double) len) /
 					    payload->bytes_per_sample);
 			session->rtp.rcv_last_ret_ts += ts_int;
-			//g_message("ts_int=%i",ts_int);
+			//ortp_debug("ts_int=%i",ts_int);
 		}
 		else
 			ts_int = 0;
@@ -1356,7 +1392,7 @@ gint rtp_session_recv_with_ts (RtpSession * session, gchar * buffer,
 			wlen = msg_to_buf (mp, buffer, rlen);
 			buffer += wlen;
 			rlen -= wlen;
-			g_message ("mlen=%i wlen=%i rlen=%i", mlen, wlen,
+			ortp_debug ("mlen=%i wlen=%i rlen=%i", mlen, wlen,
 				   rlen);
 			/* do we fill all the buffer ? */
 			if (rlen > 0)
@@ -1365,11 +1401,11 @@ gint rtp_session_recv_with_ts (RtpSession * session, gchar * buffer,
 				freemsg (mp);
 				/* if we have continuous audio, try to get other packets to fill the buffer,
 				 * ie continue the loop */
-				//g_message("User buffer not filled entirely");
+				//ortp_debug("User buffer not filled entirely");
 				if (ts_int > 0)
 				{
 					time = session->rtp.rcv_last_ret_ts;
-					g_message
+					ortp_debug
 						("Need more: will ask for %i.",
 						 time);
 				}
@@ -1383,7 +1419,7 @@ gint rtp_session_recv_with_ts (RtpSession * session, gchar * buffer,
 						       mp->b_rptr);
 				/* not enough space in the user supplied buffer */
 				/* we re-enqueue the msg with its updated read pointers for next time */
-				g_message ("Re-enqueuing packet.");
+				ortp_debug ("Re-enqueuing packet.");
 				rtp_session_lock (session);
 				rtp_putq (session->rtp.rq, mp);
 				rtp_session_unlock (session);
@@ -1447,7 +1483,7 @@ guint32 rtp_session_get_current_send_ts(RtpSession *session)
 	guint32 session_time;
 	RtpScheduler *sched=ortp_get_scheduler();
 	PayloadType *payload;
-	g_return_val_if_fail (session->payload_type>=127, 0);
+	g_return_val_if_fail (session->payload_type<128, 0);
 	payload=rtp_profile_get_payload(session->profile,session->payload_type);
 	g_return_val_if_fail(payload!=NULL, 0);
 	if ( (session->flags & RTP_SESSION_SCHEDULED)==0 ){
@@ -1460,6 +1496,24 @@ guint32 rtp_session_get_current_send_ts(RtpSession *session)
 	return userts;
 }
 
+guint32 rtp_session_get_current_recv_ts(RtpSession *session){
+	guint32 userts;
+	guint32 session_time;
+	RtpScheduler *sched=ortp_get_scheduler();
+	PayloadType *payload;
+	g_return_val_if_fail (session->payload_type<128, 0);
+	payload=rtp_profile_get_payload(session->profile,session->payload_type);
+	g_return_val_if_fail(payload!=NULL, 0);
+	if ( (session->flags & RTP_SESSION_SCHEDULED)==0 ){
+		g_warning("can't guess current timestamp because session is not scheduled.");
+		return 0;
+	}
+	session_time=sched->time_-session->rtp.rcv_time_offset;
+	userts=  (guint32)( ( (gdouble)(session_time) * (gdouble) payload->clock_rate )/ 1000.0)
+				+ session->rtp.rcv_ts_offset;
+	return userts;
+}
+
 
 #ifdef TARGET_IS_HPUXKERNEL
 void rtp_session_set_timeout (RtpSession * session, guint milisec)
@@ -1486,9 +1540,7 @@ void rtp_session_uninit (RtpSession * session)
 #ifdef BUILD_SCHEDULER
 	if (session->flags & RTP_SESSION_SCHEDULED)
 	{
-		if ((!(session->flags & RTP_SESSION_RECV_NOT_STARTED))
-			|| (!(session->flags & RTP_SESSION_SEND_NOT_STARTED)))
-                     rtp_scheduler_remove_session (session->sched,session);
+		rtp_scheduler_remove_session (session->sched,session);
 	}
 #endif
 	/*flush all queues */
@@ -1604,7 +1656,7 @@ void rtp_session_process (RtpSession * session, guint32 time, RtpScheduler *sche
 	/* send all packets that need to be sent */
 	while (cond)
 	{
-		//g_message("GRMGIMIM");
+		//ortp_debug("GRMGIMIM");
 		if (wq->q_first != NULL)
 		{
 			hdr = (rtp_header_t *) wq->q_first->b_rptr;
@@ -1614,9 +1666,9 @@ void rtp_session_process (RtpSession * session, guint32 time, RtpScheduler *sche
 						     session->rtp.
 						     snd_ts_offset) +
 				session->rtp.snd_time_offset;
-			/*g_message("session->rtp.snd_time_offset= %i, time= %i, packet_time= %i", 
+			/*ortp_debug("session->rtp.snd_time_offset= %i, time= %i, packet_time= %i", 
 				 session->rtp.snd_time_offset, time, packet_time); 
-			g_message("seeing packet seq=%i ts=%i",hdr->seq_number,hdr->timestamp);*/
+			ortp_debug("seeing packet seq=%i ts=%i",hdr->seq_number,hdr->timestamp);*/
 			if (TIME_IS_NEWER_THAN (time, packet_time))
 			{
 				mblk_t *mp;
@@ -1634,9 +1686,9 @@ void rtp_session_process (RtpSession * session, guint32 time, RtpScheduler *sche
 		}
 	}
 	/* and then try to recv packets */
-	rtp_stack_recv (session);
+	rtp_recv (session);
 	
-	//g_message("after recv");
+	//ortp_debug("after recv");
 
 	/*if we are in blocking mode or in _select(), we must wake up (or at least notify)
 	 * the application process, if its last
@@ -1659,7 +1711,7 @@ void rtp_session_process (RtpSession * session, guint32 time, RtpScheduler *sche
 
 	if (!(session->flags & RTP_SESSION_RECV_NOT_STARTED))
 	{
-		//g_message("unblocking..");
+		//ortp_debug("unblocking..");
 		/* and also wake up the application if the timestamp it asked has expired */
 		last_recv_time =
 			rtp_session_ts_to_t (session,
@@ -1667,7 +1719,7 @@ void rtp_session_process (RtpSession * session, guint32 time, RtpScheduler *sche
 					     session->rtp.
 					     rcv_query_ts_offset) +
 			session->rtp.rcv_time_offset;
-		//g_message("time=%i, last_recv_time=%i",time,last_recv_time);
+		//ortp_debug("time=%i, last_recv_time=%i",time,last_recv_time);
 		if TIME_IS_NEWER_THAN
 			(time, last_recv_time)
 		{
@@ -1675,7 +1727,7 @@ void rtp_session_process (RtpSession * session, guint32 time, RtpScheduler *sche
 			session_set_set(&sched->r_sessions,session);
 			if (session->flags & RTP_SESSION_BLOCKING_MODE)
 			{
-				//g_message("rtp_session_process: Unlocking.");
+				//ortp_debug("rtp_session_process: Unlocking.");
 				g_mutex_lock (session->rtp.
 					      wait_for_packet_to_be_recv_mutex);
 				g_cond_signal (session->rtp.
@@ -1696,5 +1748,154 @@ void rtp_add_csrc(mblk_t *mp, guint32 csrc)
 {
 	rtp_header_t *hdr=(rtp_header_t*)mp->b_rptr;
 	hdr->csrc[hdr->cc]=csrc;
+    hdr->cc++;
+}
+	
+#ifdef RTCP 
+static chunk_item_t *chunk_item_new()
+{    
+	chunk_item_t *chunk=g_new0(chunk_item_t, 1);    
+	chunk->sdes_items=g_byte_array_new();    
+	return chunk;
+}
+
+static void chunk_item_free(chunk_item_t *chunk)
+{    
+	g_byte_array_free(chunk->sdes_items, TRUE);    
+	g_free(chunk);
+}
+
+static void rtcp_add_sdes_item(chunk_item_t *chunk, rtcp_sdes_type_t sdes_type, gchar *content)
+{	
+	if ( content )
+	{
+		guint8 stype = sdes_type;
+		guint8 content_len = g_utf8_strlen(content, RTCP_SDES_MAX_STRING_SIZE);
+		g_byte_array_append(chunk->sdes_items, &stype, 1);
+		g_byte_array_append(chunk->sdes_items, &content_len, 1);
+		g_byte_array_append(chunk->sdes_items, (guint8*)content, content_len);
+	}
+}
+
+static guint rtcp_calculate_sdes_padding(guint chunk_size)
+{
+        chunk_size = chunk_size%4;
+        /* if  no rest,  return 4 */
+        if (chunk_size == 0)
+        {
+            chunk_size = 4;
+        }
+        return chunk_size;
+}
+
+static void rtcp_add_sdes_padding(chunk_item_t *chunk)
+{
+	guint8 pad[] = {0,0,0,0};
+	guint8 pad_size = rtcp_calculate_sdes_padding(chunk->sdes_items->len);
+    g_byte_array_append(chunk->sdes_items, pad, pad_size);
+}
+
+	hdr->csrc[hdr->cc]=csrc;
 	hdr->cc++;
 }
+
+void
+rtp_session_add_contributing_source(RtpSession *session, guint32 csrc, 
+    gchar *cname, gchar *name, gchar *email, gchar *phone, 
+    gchar *loc, gchar *tool, gchar *note)
+{
+	chunk_item_t *chunk = chunk_item_new(); 
+	
+	if (!cname  || !chunk)
+	{
+		g_error("Error");
+	}
+	chunk->csrc = csrc;
+	rtcp_add_sdes_item(chunk, RTCP_SDES_CNAME, cname);
+	rtcp_add_sdes_item(chunk, RTCP_SDES_NAME, name);
+	rtcp_add_sdes_item(chunk, RTCP_SDES_EMAIL, email);
+	rtcp_add_sdes_item(chunk, RTCP_SDES_PHONE, phone);
+	rtcp_add_sdes_item(chunk, RTCP_SDES_LOC, loc);
+	rtcp_add_sdes_item(chunk, RTCP_SDES_TOOL, tool);
+	rtcp_add_sdes_item(chunk, RTCP_SDES_NOTE, note);
+	rtcp_add_sdes_padding(chunk);
+	
+	g_list_append(session->contributing_sources, chunk);
+}
+
+static void rtcp_calculate_sdes_size(gpointer chunk, gpointer size)
+{
+        chunk_item_t* c = chunk;
+        guint16 *s = size;
+        *s += (c->sdes_items->len + sizeof(c->csrc));
+}
+
+static void rtcp_concatenate_sdes_item(gpointer chunk, gpointer mp)
+{
+        chunk_item_t *c = chunk;
+        mblk_t *m = mp;
+        memcpy(m->b_wptr, &c->csrc, sizeof(c->csrc));
+        m->b_wptr += sizeof(c->csrc);
+        memcpy(m->b_wptr, c->sdes_items->data, c->sdes_items->len);
+        m->b_wptr += c->sdes_items->len;
+}
+
+mblk_t* rtp_session_create_rtcp_sdes_packet(RtpSession *session)
+{
+    mblk_t *mp;
+	rtcp_common_header_t *rtcp;
+    guint16 sdes_size = 0;
+        
+    g_list_foreach(session->contributing_sources, rtcp_calculate_sdes_size, &sdes_size);
+	
+	sdes_size += RTCP_COMMON_HEADER_SIZE;
+	mp = allocb(sdes_size, 0);
+    rtcp = (rtcp_common_header_t*)mp->b_rptr;
+    rtcp->version = 2;
+    rtcp->padbit = 0;
+    rtcp->packet_type = RTCP_SDES;
+    /*maybe need a cast to guint16 FIXME*/
+	/* As in rfc1889 length is in 32-bit words minus 1*/
+    rtcp->length = (sdes_size/4)-1;
+    /*FIXME need to add rc
+	rtcp->rc=*/
+    mp->b_wptr += RTCP_COMMON_HEADER_SIZE;
+    
+    g_list_foreach(session->contributing_sources, rtcp_concatenate_sdes_item, mp);
+	mp->b_wptr += sdes_size;
+    return mp;
+}
+static gint cmp_ssrc (gconstpointer a, gconstpointer b)
+{
+	const chunk_item_t *const c = a;
+	const guint *const s = b;
+	return (c->csrc - *s);
+}       
+
+static mblk_t *rtcp_create_bye_packet(guint ssrc, gchar *reason)
+{	
+    guint packet_size = sizeof(rtcp_common_header_t);
+	mblk_t *mp = allocb(packet_size, 0);
+    rtcp_common_header_t *rtcp = (rtcp_common_header_t*)mp->b_rptr;
+
+    rtcp->version = 2;
+    rtcp->padbit = 0;
+    rtcp->packet_type = RTCP_BYE;
+    /*maybe need a cast to guint16 FIXME*/
+    rtcp->length = 1;
+	rtcp->rc = 1;
+	rtcp->ssrc = ssrc;
+    mp->b_wptr += packet_size;
+	return mp;
+}
+
+mblk_t *rtp_session_remove_contributing_sources(RtpSession *session, guint32 ssrc)
+{
+	GList *deleting = g_list_find_custom(session->contributing_sources, &ssrc, cmp_ssrc);
+	
+	chunk_item_free((chunk_item_t*)deleting->data);
+    session->contributing_sources = g_list_delete_link(session->contributing_sources, deleting);        
+	return rtcp_create_bye_packet(ssrc, NULL);
+}
+
+#endif
