@@ -117,7 +117,7 @@ void putq(queue_t *q,mblk_t *mp)
 	mp->b_next=NULL;
 	if (oldlast!=NULL) oldlast->b_next=mp;
 	else q->q_first=mp;
-	q->q_count+=mp->b_wptr-mp->b_rptr;
+	q->q_mcount++;
 }
 
 mblk_t *getq(queue_t *q)
@@ -136,7 +136,7 @@ mblk_t *getq(queue_t *q)
 		q->q_last=NULL;
 	}
 	oldfirst->b_prev=oldfirst->b_next=NULL;
-	q->q_count-=oldfirst->b_wptr-oldfirst->b_rptr;
+	q->q_mcount--;
 	return oldfirst;
 }
 
@@ -146,9 +146,9 @@ void insq(queue_t *q,mblk_t *emp, mblk_t *mp)
 	mblk_t *m;
 	
 	g_return_if_fail(mp!=NULL);
+	q->q_mcount++;
 	if (q->q_first==NULL) {
 		q->q_first=q->q_last=mp;
-	    q->q_count+=mp->b_wptr-mp->b_rptr;
 	    return;
 	}
 	if (emp==NULL){ /* insert it at the bottom of the q */
@@ -167,7 +167,6 @@ void insq(queue_t *q,mblk_t *emp, mblk_t *mp)
 			m->b_next=mp;
 		} else q->q_first=mp;
 	}
-	q->q_count+=mp->b_wptr-mp->b_rptr;
 }
 
 /* remove and free all messages in the q */
@@ -251,11 +250,3 @@ mblk_t *copymsg(mblk_t *mp)
 }
 
 #endif
-
-
-
-
-
-
-
-
