@@ -574,7 +574,7 @@ osip_dialog_register_with_authentication (OsipDialog * call_leg,
 	int i;
 	RegistrationCtxt *regctx;
 	char *uri;
-	char *pass;
+	char *pass,*p;
 	OsipUA *ua = osip_dialog_get_ua (call_leg);
 
 	if (passwd == NULL)
@@ -648,7 +648,7 @@ osip_dialog_register_with_authentication (OsipDialog * call_leg,
 	 */
 	/* !TODO */
 
-	list_add (sipmesg->authorizations, aut, -1);
+	if (aut!=NULL) list_add (sipmesg->authorizations, aut, -1);
 	if (proxy_aut != NULL)
 		list_add (sipmesg->proxy_authorizations, proxy_aut, -1);
 
@@ -721,6 +721,8 @@ osip_dialog_invite (OsipDialog * call_leg, char *callee, char *body_mime)
 			    ("error: could not build sip message.\n"));
 		return (-1);
 	}
+	printf("adding sdp body...\n");
+	fflush(NULL);
 	/* add the body */
 	if (body_mime != NULL)
 		osip_dialog_add_body_to_request (call_leg, sipmesg,
@@ -857,7 +859,8 @@ osip_dialog_reinvite_with_authentication (OsipDialog * call_leg,
 	/* !TODO */
 
 
-	list_add (sipmesg->authorizations, aut, -1);
+	if (aut!=NULL) 
+		list_add (sipmesg->authorizations, aut, -1);
 	if (proxy_aut != NULL)
 		list_add (sipmesg->proxy_authorizations, proxy_aut, -1);
 	/* !ADD AUTHENTICATION STUFF.... */
@@ -975,19 +978,16 @@ dialog_fill_route_set (dialog_t * dialog, sip_t * request)
 	 * with bis-08) then the rquri should contains the remote target
 	 * URI */
 	int i;
-	int pos = 0;
+	int pos = 0, first = 0 ;
 	url_param_t *lr_param;
 	route_t *route;
 	char *last_route;
 	/* AMD bug: fixed 17/06/2002 */
 
+	/* The route_set is stored backwards in CALLER */
 	if (dialog->type == CALLER)
-	{
-		pos = list_size (dialog->route_set) - 1;
-		route = (route_t *) list_get (dialog->route_set, pos);
-	}
-	else
-		route = (route_t *) list_get (dialog->route_set, 0);
+		first = list_size (dialog->route_set) - 1;
+	route = (route_t *) list_get (dialog->route_set, first);
 
 	url_uparam_getbyname (route->url, "lr", &lr_param);
 	if (lr_param != NULL)	/* the remote target URI is the rquri! */
@@ -1037,13 +1037,17 @@ dialog_fill_route_set (dialog_t * dialog, sip_t * request)
 			return -1;
 		if (dialog->type == CALLER)
 		{
-			if (pos != 0)
+			if (pos != first)
 				list_add (request->routes, route2, 0);
+			else
+				route_free(route2);
 		}
 		else
 		{
 			if (!list_eol (dialog->route_set, pos + 1))
 				list_add (request->routes, route2, -1);
+			else
+				route_free(route2);
 		}
 		pos++;
 	}
@@ -1660,7 +1664,24 @@ generating_request_out_of_dialog (OsipDialog * call_leg,
 
 #ifdef INET6
 	{
+		struct addrinfo hints, *res = NULL;
 		char *tmp = (char *) smalloc (90 * sizeof (char));
+		int err;
+
+		memset(&hints, 0, sizeof(hints));
+		hints.ai_family = PF_UNSPEC;
+		hints.ai_socktype = SOCK_DGRAM;
+		hints.ai_flags = AI_NUMERICHOST;
+		err = getaddrinfo(localip, NULL, &hints, &res);
+		if (err) {
+			osip_trace(OSIP_ERROR, ("This should not happen: localip %s is non-numeric\n", localip));
+			return -1;
+		}
+
+		ua->ua_family = res->ai_family;
+		/*osip_trace(OSIP_INFO1, ("%s: setting ua->ua_family=%d from localip %s\n", 
+					__FUNCTION__, ua->ua_family, localip));
+		*/
 		switch (ua->ua_family)
 		{
 		case AF_INET:
@@ -1836,12 +1857,13 @@ osip_dialog_send_request (OsipDialog * call_leg, sip_t * sipmsg)
 #else
   	if (inet_addr (dest) == -1)
 #endif
+	{
 		return (async_resolv_and_send
 			(ua->manager, transaction, sipevent));
+	}
 #ifdef INET6
 	else freeaddrinfo(res);
 #endif
-
 	ua_transaction_execute (transaction, sipevent);
 	return transaction->transactionid;
 }
