@@ -229,11 +229,12 @@ sdp_handler_add_payload (SdpHandler * sh, SdpContext * ctx,
 	}
 	if (payload->b_as_bandwidth != 0)
 	{
-		attr_field =
-			sstrdup_sprintf ("%i %i", payload->pt,
-					 payload->b_as_bandwidth);
-		sdp_b_bandwidth_add (offer, payload->line, sgetcopy ("AS"),
+		if (sdp_bandwidth_get(offer,payload->line,0)==NULL){
+			attr_field =
+			sstrdup_sprintf ("%i", payload->b_as_bandwidth);
+			sdp_b_bandwidth_add (offer, payload->line, sgetcopy ("AS"),
 				     attr_field);
+		}
 	}
 }
 
@@ -278,7 +279,7 @@ sdp_handler_generate_answer (SdpHandler * sdph, struct _SdpContext * ctx)
 {
 	sdp_t *remote = ctx->remote;
 	sdp_t *answer;
-	char *mtype, *tmp;
+	char *mtype, *tmp, *bw;
 	char *proto, *port, *pt;
 	int i, j, ncodec, m_lines_accepted = 0;
 	int err;
@@ -417,11 +418,15 @@ sdp_handler_generate_answer (SdpHandler * sdph, struct _SdpContext * ctx)
 		payload.c_addr = sdp_c_addr_get (remote, i, 0);
 		if (payload.c_addr == NULL)
 			payload.c_addr = sdp_c_addr_get (remote, -1, 0);
+		/* get application specific bandwidth, if any */
+		bw=sdp_b_bandwidth_get(remote, i, 0);
+		if (bw!=NULL) payload.b_as_bandwidth=atoi(bw);
 		if (keywordcmp ("audio", mtype) == 0)
 		{
 			if (sdph->accept_audio_codecs != NULL)
 			{
 				ncodec = 0;
+				
 				/* for each payload type */
 				for (j = 0;
 				     ((pt =
@@ -439,10 +444,7 @@ sdp_handler_generate_answer (SdpHandler * sdph, struct _SdpContext * ctx)
 						sdp_a_attr_value_get_with_pt
 						(remote, i, payload.pt,
 						 "fmtp");
-					/* get application specific bandwidth, if any */
-					payload.b_as_bandwidth =
-						sdp_b_bandwidth_get_with_pt
-						(remote, i, payload.pt);
+					
 					/* ask the application if this codec is supported */
 					err = sdph->accept_audio_codecs (sdph,
 									 ctx,
@@ -497,13 +499,13 @@ sdp_handler_generate_answer (SdpHandler * sdph, struct _SdpContext * ctx)
 						if (payload.b_as_bandwidth !=
 						    0)
 						{
-							sdp_b_bandwidth_add
+							if (sdp_bandwidth_get(answer,i,0)==NULL)
+								sdp_b_bandwidth_add
 								(answer, i,
 								 sgetcopy
 								 ("AS"),
 								 sstrdup_sprintf
-								 ("%i %i",
-								  payload.pt,
+								 ("%i",
 								  payload.
 								  b_as_bandwidth));
 						}
@@ -634,7 +636,7 @@ void
 sdp_handler_read_remote_answer (SdpHandler * sdph, struct _SdpContext *ctx)
 {
 	sdp_t *remote = ctx->remote;
-	char *mtype, *tmp;
+	char *mtype, *tmp, *bw;
 	char *proto, *port, *pt;
 	int i, j;
 	int err;
@@ -655,6 +657,9 @@ sdp_handler_read_remote_answer (SdpHandler * sdph, struct _SdpContext *ctx)
 		payload.c_addr = sdp_c_addr_get (remote, i, 0);
 		if (payload.c_addr == NULL)
 			payload.c_addr = sdp_c_addr_get (remote, -1, 0);
+		/* get application specific bandwidth, if any */
+		bw=sdp_b_bandwidth_get(remote, i, 0);
+		if (bw!=NULL) payload.b_as_bandwidth=atoi(bw);
 		if (keywordcmp ("audio", mtype) == 0)
 		{
 			if (sdph->get_audio_codecs != NULL)
@@ -705,7 +710,7 @@ sdp_handler_read_remote_answer (SdpHandler * sdph, struct _SdpContext *ctx)
 						(remote, i, payload.pt,
 						 "fmtp");
 					/* ask the application if this codec is supported */
-					err = sdph->accept_video_codecs (sdph,
+					err = sdph->get_video_codecs (sdph,
 									 ctx,
 									 &payload);
 				}
